package models;

public class TrafficModel {

	public enum Direction {
		NORTH_SOUTH, EAST_WEST
	}

	public enum TrafficState {
		GOING, SLOWING_DOWN, STOPPED
	}

	public enum PedestrianState {
		GOING, WARNING, STOPPED
	}

	// state of traffic flow
	private Direction currentDirOfTraffic = Direction.NORTH_SOUTH;
	private TrafficState northSouthTraffic = TrafficState.GOING;
	private TrafficState eastWestTraffic = TrafficState.STOPPED;
	private PedestrianState northSouthPeds = PedestrianState.GOING;
	private PedestrianState eastWestPeds = PedestrianState.STOPPED;

	// cars waiting in each direction
	public int carsWaiting = 0;

	// pedestrians waiting
	public int pedestriansWaiting = 0;

	public void addCar(Direction dir) {
		if (dir != currentDirOfTraffic) {
			++carsWaiting;
		}
	}

	public void addPedestrian(Direction dir) {
		if (dir != currentDirOfTraffic) {
			++pedestriansWaiting;
		}
	}

	public void toggleDirection() {
		if (currentDirOfTraffic == Direction.NORTH_SOUTH) {
			currentDirOfTraffic = Direction.EAST_WEST;
			eastWestTraffic = TrafficState.GOING;
			eastWestPeds = PedestrianState.GOING;
		} else {
			currentDirOfTraffic = Direction.NORTH_SOUTH;
			northSouthTraffic = TrafficState.GOING;
			northSouthPeds = PedestrianState.GOING;
		}
		pedestriansWaiting = 0;
		carsWaiting = 0;
	}

	public Direction currentFlow() {
		return currentDirOfTraffic;
	}

	public TrafficState getTrafficState(Direction d) {
		switch (d) {
		case NORTH_SOUTH:
			return northSouthTraffic;
		case EAST_WEST:
			return eastWestTraffic;
		default:
			throw new IllegalArgumentException("Bad direction");
		}
	}

	public PedestrianState getPedestrianState(Direction d) {
		switch (d) {
		case NORTH_SOUTH:
			return northSouthPeds;
		case EAST_WEST:
			return eastWestPeds;
		}
		throw new IllegalArgumentException("Bad direction");
	}

	// walk -> flashing don't walk
	public void triggerPedestrianWarning() {
		switch (this.currentDirOfTraffic) {
		case NORTH_SOUTH:
			northSouthPeds = PedestrianState.WARNING;
			break;
		case EAST_WEST:
			eastWestPeds = PedestrianState.WARNING;
			break;
		}
	}

	// green -> yellow
	public void triggerTrafficWarning() {
		switch (this.currentDirOfTraffic) {
		case NORTH_SOUTH:
			northSouthTraffic = TrafficState.SLOWING_DOWN;
			break;
		case EAST_WEST:
			eastWestTraffic = TrafficState.SLOWING_DOWN;
			break;
		}
	}

	// flashing don't walk -> don't walk
	// yellow -> red
	public void triggerStop() {
		switch (this.currentDirOfTraffic) {
		case NORTH_SOUTH:
			northSouthTraffic = TrafficState.STOPPED;
			northSouthPeds = PedestrianState.STOPPED;
			break;
		case EAST_WEST:
			eastWestTraffic = TrafficState.STOPPED;
			eastWestPeds = PedestrianState.STOPPED;
			break;
		}
	}

	/*
	 * 
	 * | | |v | ----+ +---- < > ----+ +---- | ^| | |
	 */

	//
}
